import zope.interface
import zope.component
import zope.schema
import z3c.form.interfaces
from z3c.form.widget import Widget
from z3c.form.browser import widget
from z3c.form import converter
import zope.i18nmessageid
from zope.i18n import translate

_ = zope.i18nmessageid.MessageFactory("zope")

import isequence as interfaces 


EXTRACTION = object()
PRINT = object()
        

class BaseSequenceWidget(widget.HTMLInputWidget, Widget):

    zope.interface.implements(interfaces.IBaseSequenceWidget)

    # properties that must be updated when update() is called
    need_delete = False
    need_add = False
    widgets = []
    count = 0

    def addButtonLabel(self):
        """Return localized title for the add button."""
        button_label = _('Add %s')
        button_label = translate(button_label, context=self.request,
                                 default=button_label)
        title = getattr(self.value_type, 'title', _(u"an item"))
        title = translate(title, context=self.request)
        return button_label % title

    _adapterValueAttributes = () # see z3c.form.widget.Widget

    def __init__(self, field, value_type, request):
        self.field = field
        self.value_type = value_type
        self.request = request
        # set some values displayed in the form
        self.name = field.__name__
        self.id = field.__name__.replace('.', '-')
        self.label = field.title
        self.required = field.required


    def extract(self, default=z3c.form.interfaces.NOVALUE):
        # Step 1: get count value
        try:
            _count = int(self.request.form[self.name+'.count'])
            # catch all Exceptions (conversion to integer an key error)
        except Exception, err:
            # OK to return default here?
            #raise Exception, 'count error'
            return default
        self._setupWidgets(EXTRACTION, _count)
        # Step 2: extract values from from widgets
        values = []
        deletion = self.name+'.remove' in self.request.form
        for i in range(_count):
            # Step 2.1: process DELETE
            if not (deletion and self.name+'.remove_'+str(i) in self.request.form): 
                values.append(self.widgets[i].extract())
        # Step 3: Process ADD
        if self.name+'.add' in self.request.form:
            default_value = getattr(self.value_type, 'default', None)
            values.append(default_value)
        # Step 4: return values
        return values

    def update(self):
        # Step I: let z3c.form.widget.Widget.update() work for us
        super(BaseSequenceWidget, self).update()
        # Step II: determine count from length of self.value sequence,
        # which is needed a) for the setupwidgets() method and b) in
        # the template
        if self.value not in (None,):
            self.count = len(self.value)
        else:
            # We still don't have a value, so we set count to
            # min_length from the field and generate a list of
            # min_length default values
            self.count = self.field.min_length
            defaults = []
            for i in range(self.count):
                defaults.append(getattr(self.value_type, 'default', None))
            self.value = defaults

        # Step III: set up widgets form value that we now have
        self._setupWidgets(PRINT, self.count, values = self.value)
        # Step IV: check min and max length and set need_add need_delete
        if not self.field.max_length:
            self.need_add = True
        else:
            if self.count < self.field.max_length:
                self.need_add = True
        if self.count > 0 and self.count > self.field.min_length:
            self.need_delete = True

    def _setupWidgets(self, mode, count, values = None):
        """ Setup the subwidgets.

        mode is ether EXTRACTION or PRINT. In both modes count is
        supposed to be an integer for the count of subwidgets. In
        PRINT mode values is supposed to a list type of the values to
        be printed out in the widgets."""
        # get a widget factory
        self.value_type.__name__ = 'initial' # required for z3c.form.field.Fields
        value_field = z3c.form.field.Fields(self.value_type).values()[0]
        factory = value_field.widgetFactory.get(self.mode)
        # setup widgets
        self.widgets = []
        for i in range(count):
            if factory is not None:
                widget = factory(value_field.field, self.request)
            else:
                # TODO: is there a way to look up the component only
                # once?
                widget = zope.component.getMultiAdapter(
                    (value_field.field, self.request),
                    z3c.form.interfaces.IFieldWidget)
            widget.name = self.name+"."+str(i)
            if mode == PRINT:
## this is done by z3c.form.widget.Widget.update() -- self.value has
## always passed a converter or has yust been extracted from the widget
##                 if i == 0:
##                     # look up the converter only once
##                     converter = z3c.form.interfaces.IDataConverter(widget)
##                 widget.value = converter.toWidgetValue(values[i])
                widget.value = values[i]
            widget.mode = self.mode
            widget.update()
            self.widgets.append(widget)


class TupleSequenceWidget(BaseSequenceWidget):

    zope.interface.implementsOnly(interfaces.ITupleSequenceWidget)

    zope.component.adapts(zope.schema.interfaces.ITuple,
                          zope.schema.interfaces.IField,
                          zope.publisher.interfaces.browser.IBrowserRequest
                          )


class ListSequenceWidget(BaseSequenceWidget):

    zope.interface.implementsOnly(interfaces.IListSequenceWidget)

    zope.component.adapts(zope.schema.interfaces.IList,
                          zope.schema.interfaces.IField,
                          zope.publisher.interfaces.browser.IBrowserRequest
                          )


class BaseSequenceDataConverter(converter.BaseDataConverter):

    _type = list

    def toWidgetValue(self, value):
        """See IDataConverter"""
        # since BaseSequenceWidget works with list internally we
        # convert to a list
        if value is self.field.missing_value:
            return []
        try:
            l = list(value)
        except Exception, err:
            raise ValueError, err
        return l

    def toFieldValue(self, value):
        """See IDataConverter"""
        if value == []:
            return self.field.missing_value
        # First we have to get a converter for the subwidgets, and
        # therefore we have to get a subwidget, because converters are
        # registered for widgets:
        self.widget.value_type.__name__ = 'initial' # required for z3c.form.field.Fields
        value_field = z3c.form.field.Fields(self.widget.value_type).values()[0]
        factory = value_field.widgetFactory.get(self.widget.mode)
        if factory is not None:
            subwidget = factory(value_field.field, self.widget.request)
        else:
            subwidget = zope.component.getMultiAdapter(
                (value_field.field, self.widget.request),
                z3c.form.interfaces.IFieldWidget)
        converter = z3c.form.interfaces.IDataConverter(subwidget)
        # now convert all subwidget values to field values
        values = []
        errors = []
        for i in range(len(value)):
            try:
                val = converter.toFieldValue(value[i])
                values.append(val)
            except Exception, err:
                errors.append(err)
        if errors:
            # TODO: only one error can be displayed. Or should we call
            # __repr__() on all of them, concatenate them and then raise
            # SomeException(concatenated_messages)?
            raise errors[0]
        # convert to _type
        values = self._type(values)
        # assert min_length <= len(value) <= max_length etc.
        self.field.validate(values)
        return values


class ListSequenceDataConverter(BaseSequenceDataConverter):

    _type = list

    zope.component.adapts(
        zope.schema.interfaces.IList, interfaces.IListSequenceWidget)


@zope.component.adapter(interfaces.IListSequenceWidget)
@zope.interface.implementer(z3c.form.interfaces.IDataConverter)
def ListSequenceWidgetDataConverter(widget):
    """Provide a data converter based on a field
    widget. IListSequenceWidget inherits IFieldWidget."""
    return zope.component.queryMultiAdapter(
        (widget.field, widget), z3c.form.interfaces.IDataConverter)


class TupleSequenceDataConverter(BaseSequenceDataConverter):

    _type = tuple

    zope.component.adapts(
        zope.schema.interfaces.ITuple, interfaces.ITupleSequenceWidget)


@zope.component.adapter(interfaces.ITupleSequenceWidget)
@zope.interface.implementer(z3c.form.interfaces.IDataConverter)
def TupleSequenceWidgetDataConverter(widget):
    """Provide a data converter based on a field
    widget. IListSequenceWidget inherits IFieldWidget."""
    return zope.component.queryMultiAdapter(
        (widget.field, widget), z3c.form.interfaces.IDataConverter)
