import zope.interface
import zope.schema
from persistent import Persistent
from z3c.form import field
from z3c.formui import form
from z3c.formui.interfaces import IDivFormLayer
from z3c.form.interfaces import IFormLayer
from z3c.layer.pagelet import IPageletBrowserLayer
from z3c.pagelet.browser import BrowserPagelet
from zope.traversing.browser import absoluteURL


class IIntegerTuple(zope.interface.Interface):

    integers = zope.schema.Tuple(
        title = u"Tuple of integers",
        description = u"A tuple if integers between 0 and 65535", 
        value_type = zope.schema.Int(title = u"Integer",
                                     required = True,
                                     min = 0,
                                     max = 65535,
                                     #default = 1,# TODO: if default is defined, the default value is displayed instead of the actual value in edit and display forms.
                                     ),
        min_length = 1,
        max_length = 5,
        default = (0,),
        )


class IntegerTuple(Persistent):
    """An non-persistent implementation."""

    zope.interface.implements(IIntegerTuple)

    integers = zope.schema.fieldproperty.FieldProperty(IIntegerTuple['integers'])


class ISequenceDemoBrowserLayer(IFormLayer, IPageletBrowserLayer):
    """browser layer with form support."""


class ISequenceDemoBrowserSkin(IDivFormLayer, ISequenceDemoBrowserLayer):
    """browser skin using the div-based layout."""


class FrontPage(BrowserPagelet):
    """the frontpage of the demo app."""

    def getTuples(self):
        """yields all integer tuples in the context."""
        for item in self.context.values():
            if IIntegerTuple.providedBy(item):
                yield item


class IntegerTupleAddForm(form.AddForm):
    """An add form."""

    fields = field.Fields(IIntegerTuple)

    label = u"Add a Tuple"

    def create(self, data):
        integer_tuple = IntegerTuple()
        form.applyChanges(self, integer_tuple, data)
        return integer_tuple

    def add(self, obj):
        # we use the sum of the integers as name
        self._name = 0
        for i in obj.integers:
            self._name += i
        self.context[str(self._name)] = obj

    def nextURL(self):
        return absoluteURL(self.context, self.request)


class IntegerTupleDisplayForm(form.DisplayForm):

    fields = field.Fields(IIntegerTuple)

    label = u"Display Tuple"
    

class IntegerTupleEditForm(form.EditForm):

    fields = field.Fields(IIntegerTuple)

    label = u"Edit Tuple"
